-- BuffCheck_Core.lua
-- by Mithrandir1210

--[[ 
	TO DO:
		(1/26/16 @ 01:30)
		- SendAddonMessage() WoW API Function (use to talk with other players' 
			version of BuffCheck)
			
			1. Check if any other players have the addon installed
			2. If so, get their version numbers
			3. Find highest version number (and number of addons found
				with that same version number and send "new update available" 
				message to those that are lower
			4. Error check to prevent "new update available" spam
			5. ALSO, add an option to query all players for version numbers and 
				print them to the screen
			
			?. Find a way to prevent multiple addons from spamming lists of 
			missing buffs

]]

-- GENERAL INFORMATION --======================================================

-- Load addon-local variables table
local BuffCheck_Name, bc = ...
	-- BuffCheck_Name = "BuffCheck"
	-- BuffCheck = {}

-- FUNCTIONS --================================================================

-------------------------------------------------------------------------------
-- Displays a message to chat or locally to the console if no ChatType or
--   ChatType == "console" (void).
-- Param1: Message to display (string);
-- Param2 (optional): Send message in chat or console ("RAID", "SAY", "console", etc)
-- Param3 (optional): Color of text to display (only available for console)
-------------------------------------------------------------------------------
bc.Display = function(MessageString, ChatType, FontColor)
	
	-- Set color to default if no color is specified
	local FontColor = FontColor or bc.Defaults.FontColor
	
	-- if not FontColor then
		-- local FontColor = bc.Defaults.FontColor
	-- end
	
	-- If a chat type is specified, use SendChatMessage() instead of print()
	-- Display to chat
	if ChatType and ChatType ~= "console" then
	    MessageString = "[BC] " .. MessageString
        SendChatMessage(MessageString, ChatType, nil, nil)
	-- Display to console
    elseif not ChatType or ChatType == "console" then
	    MessageString = FontColor.."[BC] "..MessageString
	    print(MessageString)
    else
		MessageString = FontColor.."[BC] "..MessageString
	    print(MessageString)
	end
    
end

-------------------------------------------------------------------------------
-- Uses the 'bc.Buffs' table to display missing buffs to the proper channel (void).
-- Settings altered here:
	-- *BuffCheck_ActiveSettings.AnnounceMissing
	-- *BuffCheck_ActiveSettings.DisableOutput
-------------------------------------------------------------------------------
bc.PrintMissingBuffs = function (Channel)
	
	-- Set and error-check channel type
	if Channel == "RAID" or Channel == "raid" or Channel == "Raid" then
		Channel = "RAID"
	elseif Channel == "PARTY" or Channel == "party" or Channel == "Party" then
		Channel = "PARTY"
	elseif Channel == "SOLO" or Channel == "solo" or Channel == "Solo" then
		Channel = "SOLO"
	else
		Channel = "console"
	end
	
	-- Display to "SAY" or console based on settings
	if not BuffCheck_ActiveSettings.AnnounceMissing then
		Channel = "console"
	elseif BuffCheck_ActiveSettings.AnnounceMissing and Channel == "SOLO" then
		Channel = "SAY"
	end
	
	if not BuffCheck_ActiveSettings.DisableBuffOutput then

		-- Print list of players missing buffs (if any buffs are missing)
		local BuffType = nil
		
		if not bc.AllBuffsFound() then
			bc.Display("Checking for missing buffs...",Channel)
			for i=1, #bc.Buffs.Index do
				BuffType = tostring(bc.Buffs.Index[i])
				bc.Buffs[BuffType]["DisplayMissing"](Channel)
			end
		elseif bc.AllBuffsFound() then
			bc.Display("All buffs found.",Channel)
		else
			-- Display directly to console
			bc.Display("Something went wrong.", "console")
		end
	end

end

-------------------------------------------------------------------------------
-- Uses the 'bc.Buffs' table to check for buffs based on zone (void).
-- Calls PrintMissingBuffs().
-------------------------------------------------------------------------------
bc.CheckBuffs = function()
	
	-- Raid group
    if IsInRaid() then
        bc.NumPlayers = GetNumGroupMembers()
        
		local BuffType = nil
		
        for i=1, bc.NumPlayers do
			for j=1, #bc.Buffs.Index do
				BuffType = bc.Buffs.Index[j]
				bc.Buffs[BuffType]["Check"]("raid"..i)
			end
        end
    
        bc.PrintMissingBuffs("RAID")
    
	-- Party
	elseif not IsInRaid() and IsInGroup(LE_PARTY_CATEGORY_HOME) then
		bc.NumPlayers = GetNumGroupMembers()
        
		local BuffType = nil
		
        for i=1, bc.NumPlayers do
            for j=1, #bc.Buffs.Index do
				BuffType = bc.Buffs.Index[j]
				bc.Buffs[BuffType]["Check"]("party"..i)
			end
        end
		
		for j=1, #bc.Buffs.Index do
			BuffType = bc.Buffs.Index[j]
			bc.Buffs[BuffType]["Check"]("player")
		end
    
        bc.PrintMissingBuffs("PARTY")
    
	-- Instance group
	elseif IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then
		bc.NumPlayers = GetNumGroupMembers()
        
		local BuffType = nil
		
        for i=1, bc.NumPlayers do
            
            for j=1, #bc.Buffs.Index do
				BuffType = bc.Buffs.Index[j]
				bc.Buffs[BuffType]["Check"]("raid"..i)
			end
        end
    
        bc.PrintMissingBuffs("RAID")
	
	-- Solo or other
    else

		local BuffType = nil
		
		-- for j=1, #bc.Buffs.Index do
			-- BuffType = bc.Buffs.Index[j]
			-- bc.Buffs[BuffType]["Check"]("player")
		-- end
		
		for j=1, #bc.Buffs.Index do
			BuffType = bc.Buffs.Index[j]
			if BuffCheck_ActiveSettings.BuffEnabled[BuffType] then
				bc.Buffs[BuffType]["Check"]("player")
			end			
		end
		
        bc.PrintMissingBuffs("SOLO")
		
    end -- if IsInRaid
    
end -- CheckBuffs()

-- SLASH COMMANDS --===========================================================

bc.SlashCmdHandler = function(msg)
	if msg == "check" then
		bc.Buffs.Clear()  
	    bc.CheckBuffs()
	elseif msg == "enable" then
		BuffCheck_ActiveSettings.DisableBuffOutput = false
		bc.btnDisableBuffOutput:SetChecked(false)
		bc.Display("Missing buffs WILL be displayed.")
	elseif msg == "disable" then
		BuffCheck_ActiveSettings.DisableBuffOutput = true
		bc.btnDisableBuffOutput:SetChecked(true)
		bc.Display("Missing buffs WILL NOT be displayed.")
	elseif msg == "announce" then
		BuffCheck_ActiveSettings.AnnounceMissing = true
		bc.btnAnnounce:SetChecked(true) -- Set checkbutton state
		bc.Display("Missing buffs will be displayed in CHAT.")
	elseif msg == "noannounce" then
		BuffCheck_ActiveSettings.AnnounceMissing = false
		bc.btnAnnounce:SetChecked(false) -- Set checkbutton state
		bc.Display("Missing buffs will be displayed LOCALLY.")
	elseif msg == "readycheck" then
		BuffCheck_ActiveSettings.CheckAfterReadycheck = true
		bc.btnReadycheck:SetChecked(true) -- Set checkbutton state
		bc.Display("Buffs WILL be checked after a readycheck occurs.")
	elseif msg == "noreadycheck" then
		BuffCheck_ActiveSettings.CheckAfterReadycheck = false
		bc.btnReadycheck:SetChecked(false) -- Set checkbutton state
		bc.Display("Buffs WILL NOT be checked after a readycheck occurs.")
	elseif msg == "viewsettings" then
		bc.ViewSettings()
	elseif msg == "settings" or msg == "options" then
		InterfaceOptionsFrame_OpenToCategory(bc.panel.name)
		InterfaceOptionsFrame_OpenToCategory(bc.panel.name)
	elseif msg == "reset" or msg == "defaults" then
		bc.SetDefaults()
	elseif msg == "help" or msg == "h" then
		bc.Display(bc.Description.Title .. " v" .. bc.Description.Version)
	    bc.Display("'/bc check' - Check buffs.")
		bc.Display("'/bc enable' - ENABLE display of missing buffs.")
		bc.Display("'/bc disable' - DISABLE display of missing buffs.")
		bc.Display("'/bc announce' - Display missing buffs in CHAT.")
		bc.Display("'/bc noannounce' - Display missing buffs LOCALLY.")
		bc.Display("'/bc readycheck' - ENABLE display of missing buffs after a readycheck.")
		bc.Display("'/bc noreadycheck' - DISABLE display of missing buffs after a readycheck.")
		bc.Display("'/bc viewsettings' - Display basic settings.")
		bc.Display("'/bc settings' or '/bc options' - Open interface options.")
		bc.Display("'/bc reset' or '/bc defaults' - RESET settings to their defaults.")
		bc.Display("'/bc version' or '/bc v' - Display current version.")
	elseif msg == "test" then
	    
		bc.Display("This is for testing/debugging purposes only.") 
		-- Debug and test functions here --
		
		--print(bc.AllBuffsFound())
	elseif msg == "version" or "v" then
		bc.Display(bc.Description.Title .. " v" .. bc.Description.Version)
    else
		bc.Display("Enter '/bc help' for a command listing.")
	end
		
end -- SlashCmdHandler()

SLASH_BUFFCHECK1 = "/bc"
SLASH_BUFFCHECK2 = "/buffcheck"
SlashCmdList["BUFFCHECK"] = bc.SlashCmdHandler

-- EVENT HANDLERS --===========================================================

-------------------------------------------------------------------------------
-- Primary event handler for the following options:
	-- Buff check after readycheck completes
-------------------------------------------------------------------------------
bc.MainEventFrame:SetScript("OnEvent", function(self, event, ...)
	
	if BuffCheck_ActiveSettings.CheckAfterReadycheck and event == "READY_CHECK_FINISHED" then
		bc.Buffs.Clear()
		bc.CheckBuffs()
	end
	
end)
